# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ceilometerclient.common import base


class EventType(base.Resource):
    def __repr__(self):
        return "<Event %s>" % self._info


def object_class_str(mgr, value, loaded):
    return EventType(mgr, {"event_type": value}, loaded)


class EventTypeManager(base.Manager):

    def list(self):
        return self._list('/v2/event_types/', obj_class=object_class_str)
